import unittest
from dataclasses import dataclass

from jsonasobj import JsonObj


class Root(JsonObj):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._initialized = True
        self.__post_init__()

    def __post_init__(self):
        # leave a mark saying we've been here
        self._post_initialized = True


class A(Root):
    def __init__(self, v: int) -> None:
        super().__init__()
        self.v = v


@dataclass
class C(Root):
    v: int


class InitTestCase(unittest.TestCase):
    def test_list_or_dict(self):
        """ Make sure that only lists or dictionaries get initialized """
        with self.assertRaises(TypeError) as e:
            JsonObj(17)
        self.assertIn('JSON Object can only be a list or dictionary', str(e.exception))
        o = JsonObj(JsonObj(a=1, b={"@x": 17}))
        self.assertEqual("JsonObj(a=1, b=JsonObj(**{'@x': 17}))", str(o))

    @unittest.skip("Underscore testing turns out to be far to invasive -- checks have been removed")
    def test_no_underscores(self):
        """ Make sure that underscore values cannot be constructed or added afterwards """
        with self.assertRaises(ValueError) as e:
            JsonObj(_v1=1, x=12, _y="abc")
        self.assertIn('_v1: underscore not allowed', str(e.exception))
        with self.assertRaises(ValueError) as e:
            JsonObj(dict(a="A", b="B", c=dict(c1=12), _d="D"))
        self.assertIn('_d: underscore not allowed', str(e.exception))
        with self.assertRaises(ValueError) as e:
            JsonObj(dict(a="A", b="B", c=dict(_c1=12)))
        self.assertIn("_c1: underscore not allowed", str(e.exception))
        o = JsonObj(dict(a="A", b="B", c=dict(c1=12), d="D"))
        with self.assertRaises(ValueError) as e:
            o._e = "E"
        self.assertIn("_e: underscore not allowed", str(e.exception))
        with self.assertRaises(ValueError) as e:
            o.e = dict(_f="F")
        self.assertIn("_f: underscore not allowed", str(e.exception))
        with self.assertRaises(ValueError) as e:
            o.c._g="G"
        self.assertIn("_g: underscore not allowed", str(e.exception))

    def test_basic_init(self):
        """ JsonObj __init__ function is not getting called when invoking via a dataclass """
        x = A(v=1)
        self.assertTrue(x._post_initialized)
        self.assertTrue(x._initialized)
        self.assertEqual('A(v=1)', str(x))
        y = C(1)
        self.assertTrue(y._post_initialized)
        self.assertEqual('C(v=1)', str(y))


if __name__ == '__main__':
    unittest.main()




import unittest
import jsonasobj


class ExtendedNamespaceTestCase(unittest.TestCase):
    """ ExtendedNamespace is a dictionary / namespace combination.  Any valid python identifiers are also available
    as first class members.
    """
    def test_extendednamespace(self) -> None:
        # Direct constructor
        ens = jsonasobj.ExtendedNamespace(i1=1, i2='a', i3='17')
        # Test dictionary and namespace behavior
        self.assertEqual(1, ens.i1)
        self.assertEqual('a', ens.i2)
        self.assertEqual('17', ens.i3)
        self.assertEqual(3, len(ens))
        self.assertTrue('i1' in ens)
        self.assertFalse('i4' in ens)
        del ens['i1']
        self.assertEqual(2, len(ens))
        self.assertFalse('i1' in ens)
        ens.i4 = 243
        self.assertEqual(243, ens.i4)
        self.assertEqual(243, ens['i4'])

        # Construct from a dictionary, using common json-ld idioms
        ens1 = jsonasobj.ExtendedNamespace(**{'i1': 1, '@foo': "bar", 'x:y': ens})
        self.assertEqual(1, ens1.i1)
        self.assertEqual('bar', ens1['@foo'])
        self.assertEqual(ens, ens1['x:y'])
        self.assertEqual(3, len(ens1))

    def test_extended_access(self) -> None:
        ens = jsonasobj.ExtendedNamespace(i1=1, i2='a', i3='17')
        self.assertEqual("no", ens._get('i4', 'no'))


if __name__ == '__main__':
    unittest.main()



import unittest

import jsonasobj

test_data = {
    "@context": {
        "name": "http://xmlns.com/foaf/0.1/name",
        "knows": "http://xmlns.com/foaf/0.1/knows",
        "menu": {
            "@id": "name:foo",
            "@type": "@id"
        }
    },
    "@id": "http://me.markus-lanthaler.com/",
    "name": "Markus Lanthaler",
    "knows": [
        {
            "name": "Dave Longley",
            "menu": "something",
            "modelDate" : "01/01/2015"
        }
    ]
}
test_json = str(test_data).replace("'", '"')

expected='''{
   "name": "Markus Lanthaler",
   "knows": [
      {
         "name": "Dave Longley",
         "menu": "something",
         "modelDate": "01/01/2015"
      }
   ]
}'''


class MissingObjectTestCase(unittest.TestCase):
    def test_missing(self):
        pyobj = jsonasobj.loads(test_json)
        self.assertEqual('Markus Lanthaler', pyobj.name)
        self.assertEqual(pyobj.name, pyobj['name'])
        with self.assertRaises(AttributeError) as e:
            self.assertEqual('Markus Lanthaler', pyobj.missing_item1)
        self.assertIn("object has no attribute 'missing_item1'", str(e.exception))
        with self.assertRaises(KeyError) as e:
            pyobj['12345']
        self.assertIn("'12345'", str(e.exception))
        pyobj._if_missing = lambda obj, item: (True, f"Missing: {item}")
        self.assertEqual("Missing: missing_item1", pyobj.missing_item1)
        self.assertEqual("Missing: 12345", pyobj['12345'])


if __name__ == '__main__':
    unittest.main()




import unittest
import jsonasobj

test_json = """{
    "k1": 1,
    "k2": "abc",
    "k3": {
        "x1": "foo",
        "x2": 17
    }
}"""


class ItemsTestCase(unittest.TestCase):
    def test_items(self):
        py_obj = jsonasobj.loads(test_json)
        for k, v in py_obj._items():
            if k == 'k1':
                self.assertEqual(1, v)
            elif k == 'k2':
                self.assertEqual("abc", v)
            else:
                self.assertEqual(k, "k3")
                self.assertTrue(isinstance(v, jsonasobj.JsonObj))
                self.assertEqual("foo", v.x1)
                self.assertEqual(17, v.x2)

if __name__ == '__main__':
    unittest.main()



import unittest

import jsonasobj


test_data = {
    "@context": {
        "name": "http://xmlns.com/foaf/0.1/name",
        "knows": "http://xmlns.com/foaf/0.1/knows",
        "menu": {
            "@id": "name:foo",
            "@type": "@id"
        }
    },
    "@id": "http://me.markus-lanthaler.com/",
    "name": "Markus Lanthaler",
    "knows": [
        {
            "name": "Dave Longley",
            "menu": "something",
            "modelDate" : "01/01/2015"
        }
    ]
}
test_json = str(test_data).replace("'", '"')

expected='''{
   "name": "Markus Lanthaler",
   "knows": [
      {
         "name": "Dave Longley",
         "menu": "something",
         "modelDate": "01/01/2015"
      }
   ]
}'''


class FilterTestCase(unittest.TestCase):

    @staticmethod
    def filtr(e: dict) -> dict:
        return {k: v for k, v in e.items() if not k.startswith('@')}

    def test_filters(self):
        pyobj = jsonasobj.loads(test_json)
        self.assertEqual(expected, jsonasobj.as_json(pyobj, filtr=self.filtr))
        self.assertEqual(expected, pyobj._as_json_dumps(filtr=self.filtr))


if __name__ == '__main__':
    unittest.main()



from jsonasobj._jsonobj import JsonObj, as_dict, as_list, as_json, as_json_obj, get, items, loads, load, setdefault, \
    keys, items
from jsonasobj.extendednamespace import ExtendedNamespace

__all__ = ['JsonObj', 'ExtendedNamespace', 'as_dict', 'as_list', 'as_json', 'as_json_obj', 'get', 'items',
           'load', 'loads', 'setdefault', 'keys']



import unittest
from copy import deepcopy
from dataclasses import dataclass

from jsonasobj import JsonObj


class YAMLRoot(JsonObj):
    pass


@dataclass
class Test(YAMLRoot):
    v: int


class MissingConstructorTestCase(unittest.TestCase):
    def test_missing_constructor(self):
        """ Test for reasonable behavior when the constructor wasn't invoked """
        z = Test(1)
        deepcopy(z)     # Invokes test for local __deepcopy__ in z


if __name__ == '__main__':
    unittest.main()



import jsonasobj
from jsonasobj._jsonobj import as_json, as_dict

test_json = """{
  "@context": {
    "name": "http://xmlns.com/foaf/0.1/name",
    "knows": "http://xmlns.com/foaf/0.1/knows",
    "menu": {
      "@id": "name:foo",
      "@type": "@id"
    }
  },
  "@id": "http://me.markus-lanthaler.com/",
  "name": "Markus Lanthaler",
  "knows": [
    {
      "name": "Dave Longley",
      "menu": "something",
      "modelDate" : "01/01/2015"
    }
  ]
}"""

py_obj = jsonasobj.loads(test_json)
py_obj.knows[0].extra = {'age': 17}
py_obj.knows.append(dict(name='Barack Obama'))
del py_obj.knows[0]['menu']
print(py_obj.name)
print(py_obj['name'])
print(py_obj.knows[0].name)
print(py_obj['@context'].name)
print(as_json(py_obj))
print(as_dict(py_obj))
'''
Result:

Markus Lanthaler
Markus Lanthaler
Dave Longley
http://xmlns.com/foaf/0.1/name
{
   "@id": "http://me.markus-lanthaler.com/",
   "knows": [
      {
         "extra": {
            "age": 17
         },
         "name": "Dave Longley",
         "modelDate": "01/01/2015"
      },
      {
         "name": "Barack Obama"
      }
   ],
   "name": "Markus Lanthaler",
   "@context": {
      "menu": {
         "@id": "name:foo",
         "@type": "@id"
      },
      "knows": "http://xmlns.com/foaf/0.1/knows",
      "name": "http://xmlns.com/foaf/0.1/name"
   }
}
{'@id': 'http://me.markus-lanthaler.com/', 'knows': [{'modelDate': '01/01/2015', 'extra': {'age': 17}, 'name': 'Dave Longley'}, {'name': 'Barack Obama'}], 'name': 'Markus Lanthaler', '@context': {'menu': {'@id': 'name:foo', '@type': '@id'}, 'knows': 'http://xmlns.com/foaf/0.1/knows', 'name': 'http://xmlns.com/foaf/0.1/name'}}

'''



import unittest

from pyjsg.jsglib import JSGObjectMap, JSGString, JSGPattern, ArrayFactory, JSGContext, Integer
from pyjsg.validate_json import JSGPython

from json import loads as jsonloads
from jsonasobj import as_json

# Taken from
class HEX(JSGString):
    pattern = JSGPattern(r'[0-9]|[A-F]|[a-f]')


_CONTEXT = JSGContext()


class BuiltinSyntaxTestCase(unittest.TestCase):
    """ This tests a somewhat mysterious issue focused around the as_json function """
    # From: https://github.com/hsolbrig/pyjsg/blob/master/tests/test_issues/test_builtins_issue.py
    def test_1(self):
        x = JSGPython('''doc {
    v1: @string,
    v2: @number,
    v3: @int,
    v4: @bool,
    v5: @null,
    v6: @array,
    v7: @object 
}
obj {a: . , }''')

        rslt = x.conforms('''
        { "v1": "This is text!",
          "v2": -117.432e+2,
          "v3": -100173,
          "v4": false,
          "v5": null,
          "v6": [12, "text", null],
          "v7": {"q": "life", "a": 42}
        }''')
        self.assertTrue(rslt.success)

    def test_basic_map(self):
        """ Test sometimes failing pyjsg use case for jsonasobj """
        # From: https://github.com/hsolbrig/pyjsg/blob/master/tests/test_jsglib/test_objectmap.py#L15
        class IntObjectMap(JSGObjectMap):
            _name_filter = HEX
            _value_type = ArrayFactory('', _CONTEXT, Integer, 0, None)

            def __init__(self,
                         **_kwargs):
                super().__init__(_CONTEXT, **_kwargs)

        x = IntObjectMap()
        x.E = [1,2,3]
        self.assertTrue(x._is_valid())
        self.assertEqual(as_json(x), as_json(jsonloads('{"E":[1,2,3]}')))



import json
from typing import Union, List, Dict, Tuple, Optional, Callable, Any, Iterator
from hbreader import hbread

from jsonasobj.extendednamespace import ExtendedNamespace

# Possible types in the JsonObj representation
JsonObjTypes = Union["JsonObj", List["JsonObjTypes"], str, bool, int, float, None]

# Types in the pure JSON representation
JsonTypes = Union[Dict[str, "JsonTypes"], List["JsonTypes"], str, bool, int, float, None]

# Control variables -- note that subclasses can add to this list
hide = ['_if_missing', '_root']


class JsonObj(ExtendedNamespace):
    """ A namespace/dictionary representation of a JSON object. Any name in a JSON object that is a valid python
    identifier is represented as a first-class member of the objects.  JSON identifiers that begin with "_" are
    disallowed in this implementation.
    """
    def __new__(cls, list_or_dict: Optional[Union[List, Dict]] = None, *,
                _if_missing: Callable[["JsonObj", str], Tuple[bool, Any]] = None, **kwargs):
        """ Construct a JsonObj from set of keyword/value pairs

        :param list_or_dict: A list or dictionary that can be used to construct the object
        :param _if_missing: Function to call if attempt is made to access an undefined value.  Function takes JsonObj
        instance and parameter as input and returns a tuple -- handled (y or n) and result.  If handled is 'n' inline
        processing proceeds.
        :param kwargs: A dictionary as an alternative constructor.
        """
        # This makes JsonObj idempotent
        if isinstance(list_or_dict, JsonObj):
            if not kwargs and (not _if_missing or _if_missing == list_or_dict._if_missing):
                return list_or_dict
            else:
                obj = super(ExtendedNamespace, cls).__new__(cls)
                obj.__init__(as_json_obj(list_or_dict), _if_missing=_if_missing, **kwargs)
        else:
            obj = super(ExtendedNamespace, cls).__new__(cls)
        return obj

    def __init__(self, list_or_dict: Optional[Union[List, Dict]] = None, *,
                 _if_missing: Callable[["JsonObj", str], Tuple[bool, Any]] = None, **kwargs):
        """ Construct a JsonObj from set of keyword/value pairs

        :param list_or_dict: A list or dictionary that can be used to construct the object
        :param _if_missing: Function to call if attempt is made to access an undefined value.  Function takes JsonObj
        instance and parameter as input and returns a tuple -- handled (y or n) and result.  If handled is 'n' inline
        processing proceeds.
        :param kwargs: A dictionary as an alternative constructor.
        """
        if isinstance(list_or_dict, JsonObj):
            return

        if _if_missing and _if_missing != self._if_missing:
            self._if_missing = _if_missing
        if list_or_dict is not None:
            if kwargs:
                raise TypeError("Constructor can't have both a single item and a dict")
            if isinstance(list_or_dict, JsonObj):
                pass
            elif isinstance(list_or_dict, dict):
                self._init_from_dict(list_or_dict)
            elif isinstance(list_or_dict, list):
                ExtendedNamespace.__init__(self,
                                           _root=[JsonObj(e) if isinstance(e, (dict, list)) else
                                                  e for e in list_or_dict])
            else:
                raise TypeError("JSON Object can only be a list or dictionary")
        else:
            self._init_from_dict(kwargs)

    @staticmethod
    def _if_missing(obj: "JsonObj", item: str) -> Tuple[bool, Any]:
        return False, None

    def _init_from_dict(self, d: Union[dict, "JsonObj"]) -> None:
        """ Construct a JsonObj from a dictionary or another JsonObj """
        if not isinstance(d, JsonObj):
            ExtendedNamespace.__init__(self, _if_missing=self._if_missing,
                                       **{k: JsonObj(v) if isinstance(v, dict) else v for k, v in d.items()})

    def _hide_list(self):
        return self._root if '_root' in self else self

    def __str__(self) -> str:
        return str(self._root) if '_root' in self else super().__str__()

    # ===================================================
    # JSON Serializer method
    # ===================================================
    @staticmethod
    def _default(obj, filtr: Callable[[dict], dict] = lambda e: e):
        """ return a serialized version of obj or raise a TypeError.  Used by the JSON serializer

        :param obj:
        :param filtr: dictionary filter
        :return: Serialized version of obj
        """
        return filtr(obj._as_dict) if isinstance(obj, JsonObj) else json.JSONDecoder().decode(obj)

    # ===================================================
    # Underscore equivalent of useful dictionary functions
    # ===================================================
    def _get(self, item: str, default: JsonObjTypes = None) -> JsonObjTypes:
        """ Equivalent to dictionary get function w/o polluting namespace """
        return self[item] if item in self else default

    def _setdefault(self, k: str, value: Union[Dict, JsonTypes]) -> JsonObjTypes:
        """ Equivalent of dictionary setdefault without messing in namespace """
        if k not in self:
            self[k] = JsonObj(_if_missing=self._if_missing, **value) if isinstance(value, dict) else value
        return self[k]

    def _keys(self) -> List[str]:
        """ Return all non-hidden keys """
        for k in self._hide_list().__dict__.keys():
            if k not in hide:
                yield k

    def _items(self) -> List[Tuple[str, JsonObjTypes]]:
        """ Return all non-hidden items """
        for k, v in self._hide_list().__dict__.items():
            if k not in hide:
                yield k, v

    # ===================================================
    # Various converters -- use exposed methods in place of underscores
    # ===================================================
    def _as_json_obj(self) -> JsonTypes:
        """ Return self as pure json """
        return json.loads(self._as_json_dumps())

    def __getitem__(self, item):
        if '_root' in self:
            return self._root[item]
        else:
            found, val = self._if_missing(self, item)
            if found:
                return val
            else:
                return super().__getitem__(item)

    def __getattr__(self, item):
        found, val = self._if_missing(self, item)
        if found:
            return val
        else:
            return super().__getattribute__(item)

    def __setattr__(self, key, value):
        super().__setattr__(key, JsonObj(value) if isinstance(value, dict) else value)

    @property
    def _as_json(self) -> str:
        """ Convert a JsonObj into straight json text

        :return: JSON formatted str
        """
        return json.dumps(self, default=self._default)

    def _as_json_dumps(self, indent: str = '   ', filtr: Callable[[dict], dict] = None, **kwargs) -> str:
        """ Convert to a stringified json object.

        This is the same as _as_json with the exception that it isn't
        a property, meaning that we can actually pass arguments...
        :param indent: indent argument to dumps
        :param filtr: dictionary filter
        :param kwargs: other arguments for dumps
        :return: JSON formatted string
        """
        return json.dumps(self, default=lambda obj: self._default(obj, filtr) if filtr else self._default(obj),
                          indent=indent, **kwargs)

    @staticmethod
    def _as_list(value: List[JsonObjTypes]) -> List[JsonTypes]:
        """ Return a json array as a list

        :param value: array
        :return: array with JsonObj instances removed
        """
        return [e._as_dict if isinstance(e, JsonObj) else e for e in value]

    @property
    def _as_dict(self) -> Dict[str, JsonTypes]:
        """ Convert a JsonObj into a straight dictionary

        :return: dictionary that cooresponds to the json object
        """
        return {k: v._as_dict if isinstance(v, JsonObj) else self._as_list(v) if isinstance(v, list) else v
                for k, v in self._items()}


def loads(s: str, **kwargs) -> JsonObj:
    """ Convert a json_str into a JsonObj

    :param s: a str instance containing a JSON document
    :param kwargs: arguments see: json.load for details
    :return: JsonObj representing the json string
    """
    if isinstance(s, (bytes, bytearray)):
        s = s.decode(json.detect_encoding(s), 'surrogatepass')

    return JsonObj(json.loads(s, object_hook=lambda pairs: JsonObj(pairs), **kwargs))


def load(source, **kwargs) -> JsonObj:
    """ Deserialize a JSON source.

    :param source: a URI, File name or a .read()-supporting file-like object containing a JSON document
    :param kwargs: arguments. see: json.load for details
    :return: JsonObj representing fp
    """
    return loads(hbread(source, accept_header="application/json, text/json;q=0.9"), **kwargs)


def as_dict(obj: Union[JsonObj, List]) -> Union[List, Dict[str, JsonTypes]]:
    """ Convert a JsonObj into a straight dictionary

    :param obj: pseudo 'self'
    :return: dictionary that cooresponds to the json object
    """
    return [e._as_dict if isinstance(e, JsonObj) else e for e in obj] if isinstance(obj, list) else\
        obj._as_dict if isinstance(obj, JsonObj) else obj


def as_list(obj: Union[JsonObj, List]) -> List[JsonTypes]:
    """ Return a json array as a list

    :param obj: pseudo 'self'
    """
    return obj._as_list if isinstance(obj, JsonObj) else obj


def as_json(obj: Union[Dict, JsonObj, List], indent: Optional[str] = '   ',
            filtr: Callable[[dict], dict] = None, **kwargs) -> str:
    """ Convert obj to json string representation.

        :param obj: pseudo 'self'
        :param indent: indent argument to dumps
        :param filtr: filter to remove unwanted elements
        :param kwargs: other arguments for dumps
        :return: JSON formatted string
       """
    if isinstance(obj, JsonObj) and '_root' in obj:
        obj = obj._root
    return obj._as_json_dumps(indent, filtr=filtr, **kwargs) if isinstance(obj, JsonObj) else \
        json.dumps(obj, default=lambda o: JsonObj._default(o, filtr) if filtr else JsonObj._default(o),
                   indent=indent, **kwargs)


def as_json_obj(obj: Union[Dict, JsonObj, List]) -> JsonTypes:
    """ Return obj as pure python json (vs. JsonObj)
        :param obj: pseudo 'self'
        :return: Pure python json image
    """
    if isinstance(obj, JsonObj):
        obj = obj._hide_list()
    return [as_json_obj(e) if isinstance(e, JsonObj) else e for e in obj] if isinstance(obj, list) else\
        obj._as_json_obj() if isinstance(obj, JsonObj) else obj


def get(obj: Union[Dict, JsonObj], item: str, default: JsonObjTypes = None) -> JsonObjTypes:
    """ Dictionary get routine """
    return obj._get(item, default) if isinstance(obj, JsonObj) else obj.get(item, default)


def setdefault(obj: Union[Dict, JsonObj], k: str, value: Union[Dict, JsonTypes]) -> JsonObjTypes:
    """ Dictionary setdefault routine """
    return obj._setdefault(k, value) if isinstance(obj, JsonObj) else obj.setdefault(k, value)


def keys(obj: Union[Dict, JsonObj]) -> Iterator[str]:
    """ same as dict keys() without polluting the namespace """
    return obj._keys() if isinstance(obj, JsonObj) else obj.keys()


def items(obj: Union[Dict, JsonObj]) -> Iterator[Tuple[str, JsonObjTypes]]:
    """ Same as dict items() except that the values are JsonObjs instead of vanilla dictionaries
    :return:
    """
    return obj._items() if isinstance(obj, JsonObj) else obj.items()





class ExtendedNamespace:
    """ A combination of a namespace and a dictionary.  This allows direct access to python properties plus
    dictionary access to everything.
    """
    def __init__(self, **kwargs):
        for name in kwargs:
            setattr(self, name, kwargs[name])

    def __eq__(self, other):
        if not isinstance(other, ExtendedNamespace):
            return NotImplemented
        return vars(self) == vars(other)

    def __contains__(self, key):
        return key in self.__dict__

    def __getitem__(self, item):
        return self.__dict__[item]

    def __setitem__(self, key, item):
        self.__dict__[key] = item

    def __delitem__(self, key):
        del self.__dict__[key]

    def __iter__(self):
        return self.__dict__.__iter__()

    def __len__(self):
        return len(self.__dict__)

    def __repr__(self):
        type_name = type(self).__name__
        arg_strings = []
        star_args = {}
        for name, value in list(self.__dict__.items()):
            if not name.startswith('_'):
                if name.isidentifier():
                    arg_strings.append('%s=%r' % (name, value))
                else:
                    star_args[name] = value
        if star_args:
            arg_strings.append('**%s' % repr(star_args))
        return '%s(%s)' % (type_name, ', '.join(arg_strings))

    def _get(self, key, default=None):
        return self.__dict__.get(key, default)



import unittest
import json
import jsonasobj
from dict_compare import compare_dicts

from jsonasobj._jsonobj import as_json, as_dict

test_data = {
  "@context": {
    "name": "http://xmlns.com/foaf/0.1/name",
    "knows": "http://xmlns.com/foaf/0.1/knows",
    "menu": {
      "@id": "name:foo",
      "@type": "@id"
    }
  },
  "@id": "http://me.markus-lanthaler.com/",
  "name": "Markus Lanthaler",
  "knows": [
    {
      "name": "Dave Longley",
      "menu": "something",
      "modelDate" : "01/01/2015"
    }
  ]
}
test_json = str(test_data).replace("'", '"')

test_data_slim = {
    "knows": [{"name": "Dave Longley"}]
}
test_json_slim = str(test_data_slim).replace("'", '"')


class ExampleTestCase(unittest.TestCase):
    def test_example(self):
        pyobj = jsonasobj.loads(str(test_json))
        self.assertEqual('Markus Lanthaler', pyobj.name)
        self.assertEqual(pyobj.name, pyobj['name'])
        self.assertEqual('Dave Longley', pyobj.knows[0].name)
        self.assertEqual('http://xmlns.com/foaf/0.1/name', pyobj['@context'].name)
        self.assertEqual(json.loads(test_json), json.loads(pyobj._as_json))
        self.assertEqual(json.loads(pyobj._as_json), json.loads(as_json(pyobj)))
        self.assertTrue(compare_dicts(test_data, pyobj._as_dict))
        self.assertTrue(compare_dicts(test_data, as_dict(pyobj)))

    def test_example_slim(self):
        """ Test a slimmed down version of example for inner list """
        pyobj = jsonasobj.loads(test_json_slim)
        self.assertEqual('Dave Longley', pyobj.knows[0].name)



if __name__ == '__main__':
    unittest.main()



import json
import unittest

from jsonasobj import JsonObj, loads, as_json, as_json_obj, as_dict

s_list = '''[
   {
      "a": 17,
      "b": "sell",
      "c": [
         1,
         2,
         3
      ],
      "d": null,
      "e": {},
      "f": {
         "g": -12.56
      }
   },
   143
]'''

p_list = [{'a': 17, 'b': 'sell', 'c': [1, 2, 3], 'd': None, 'e': {}, 'f': {'g': -12.56}}, 143]


class OuterListTestCase(unittest.TestCase):
    """ Test various forms of outermost lists """
    def test_loads_list(self):
        t1 = loads(s_list)
        self.assertEqual(p_list, as_json_obj(t1))
        self.assertEqual(s_list, as_json(t1))

    def test_list_constructor(self):
        t1 = loads(s_list)
        t1_dict = as_json_obj(t1)
        self.assertEqual(p_list, t1_dict)
        t2 = JsonObj(t1_dict)
        self.assertEqual(s_list, as_json(t2))

    def test_list_subscripts(self):
        t1 = loads(s_list)
        self.assertEqual(3, t1[0].c[2])
        self.assertEqual(-12.56, t1[0].f.g)

    @unittest.skip("Setters have yet to be implemented")
    def test_list_setters(self):
        t1 = JsonObj([])
        print(t1)
        t1.append('14')
        print(as_json(t1))


if __name__ == '__main__':
    unittest.main()



import unittest
import json
from dict_compare import compare_dicts

import jsonasobj
from jsonasobj._jsonobj import as_json, as_dict

test_json = """{
  "@context": {
    "name": "http://xmlns.com/foaf/0.1/name",
    "knows": "http://xmlns.com/foaf/0.1/knows",
    "menu": {
      "@id": "name:foo",
      "@type": "@id"
    }
  },
  "@id": "http://me.markus-lanthaler.com/",
  "name": "Markus Lanthaler",
  "knows": [
    {
      "name": "Dave Longley",
      "menu": "something",
      "modelDate" : "01/01/2015"
    }
  ]
}"""


class BasicFunctionsTestCase(unittest.TestCase):
    """ Test the basic JSON access for the JSON-LD source
    """
    def test_basic_json_read(self) -> None:
        """ Test the basic JSON level read
        """
        py_obj = jsonasobj.loads(test_json)
        self.assertEqual("Markus Lanthaler", py_obj.name)
        self.assertEqual("Dave Longley", py_obj.knows[0].name)
        self.assertEqual("http://xmlns.com/foaf/0.1/name", py_obj["@context"].name)
        self.assertEqual("http://me.markus-lanthaler.com/", py_obj["@id"])

    def test_as_json(self):
        """ Test the JSON serialization
        """
        py_obj = jsonasobj.loads(test_json)
        self.assertEqual(json.loads(test_json), json.loads(py_obj._as_json))
        self.assertEqual(json.loads(test_json), json.loads(as_json(py_obj)))

    def test_setdefault(self):
        py_obj = jsonasobj.JsonObj()
        py_obj._setdefault('test', dict(foo=17))
        py_obj._setdefault('test', 'nada')
        py_obj._setdefault('test2', 'sama')
        self.assertTrue(compare_dicts(dict(test=dict(foo=17), test2="sama"), py_obj._as_dict))
        self.assertTrue(compare_dicts(dict(test=dict(foo=17), test2="sama"), as_dict(py_obj)))



import json
import unittest

import jsonasobj
from jsonasobj import JsonObj

test_data = {
    "name": "Markus Lanthaler"
}
test_json = str(test_data).replace("'", '"')


class TestObj(JsonObj):
    def _default(self, obj):
        return super()._default(obj)


class NonDefaultTestCase(unittest.TestCase):
    def test_old_default(self):
        e = TestObj(**json.loads(test_json))
        # Failure point 1
        self.assertEqual('''{
   "name": "Markus Lanthaler"
}''', jsonasobj.as_json(e))
        self.assertEqual('''{
yy"name": "Markus Lanthaler"
}''', jsonasobj.as_json(e, indent='yy'))
        # Failure point 2
        self.assertEqual('''{
   "name": "Markus Lanthaler"
}''', e._as_json_dumps())
        self.assertEqual('''{
zz"name": "Markus Lanthaler"
}''', e._as_json_dumps(indent='zz'))


if __name__ == '__main__':
    unittest.main()



import json
import os
import unittest
from typing import Callable

from dict_compare import dict_compare

from jsonasobj import as_json_obj, as_dict, as_list, as_json, get, setdefault, JsonObj, keys, items
from jsonasobj import loads as jso_loads, load as jso_load


CWD = os.path.dirname(__file__)
INPUT_DIR = os.path.join(CWD, 'input')


class NonJSONObjTestCase(unittest.TestCase):

    def _check_results(self, xjo: JsonObj, f: Callable[[JsonObj], dict]) -> None:
        self.assertTrue(isinstance(xjo, JsonObj))
        self.assertTrue(isinstance(xjo.a, JsonObj))
        self.assertTrue(isinstance(xjo['a'], JsonObj))
        self.assertTrue(isinstance(xjo.a.b, JsonObj))
        self.assertTrue(isinstance(xjo['a']['b'], JsonObj))
        self.assertEqual(17, xjo.a.b.c)
        self.assertEqual(17, xjo['a'].b['c'])

        xd = f(xjo)
        self.assertTrue(isinstance(xd, dict))
        self.assertTrue(isinstance(xd['a'], dict))
        self.assertTrue(isinstance(xd['a']['b'], dict))
        self.assertEqual(17, xd['a']['b']['c'])

    def test_deep_nesting(self):
        """ Make sure that the constructor does JsonObjs all the way down """
        self._check_results(JsonObj({"a": {"b": {"c": 17}}}), as_dict)

    def test_deep_nesting_loads(self):
        """ Make sure that the loads constructor does JsonObjs all the way down """
        self._check_results(jso_loads('{"a": {"b": {"c": 17}}}'), as_dict)

    def test_as_json_obj(self):
        """ Make sure the as_json_obj produces a pure dictionary """
        self._check_results(jso_loads('{"a": {"b": {"c": 17}}}'), as_json_obj)

    def test_non_jsonasobj_idioms(self):
        """ Make sure that the helper functions work with both JSONObjs and plain old JSON dicts """
        for fname in os.listdir(INPUT_DIR):
            if fname.endswith('.json'):
                fpath = os.path.join(INPUT_DIR, fname)
                with open(fpath) as f:
                    json_repr = json.load(f)
                jsonasobj_repr = jso_load(fpath)
                self.assertTrue(dict_compare(json_repr, as_json_obj(jsonasobj_repr)))
                self.assertTrue(dict_compare(json_repr, as_json_obj(json_repr)))
                self.assertTrue(dict_compare(as_dict(json_repr), as_dict(jsonasobj_repr)))
                self.assertTrue(dict_compare(as_dict(as_list([json_repr, json_repr])[1]),
                                             as_dict(as_list([jsonasobj_repr, jsonasobj_repr])[0])))
                self.assertTrue(dict_compare(json.loads(as_json(json_repr)), json.loads(as_json(jsonasobj_repr))))

    def test_dict_idioms(self):
        """ Test get, setdefault, keys, and items passthroughs on dicts and jsonobjs """
        fname = os.path.join(CWD, 'input', 'meta.json')
        with open(fname) as f:
            json_repr = json.load(f)
        jsonasobj_repr = jso_load(fname)
        self.assertEqual(get(jsonasobj_repr, 'license'), get(jsonasobj_repr, 'license'))
        self.assertEqual(get(jsonasobj_repr, 'subsets')[0]['name'], get(jsonasobj_repr, 'subsets')[0].name)
        setdefault(json_repr, 'z', {"a": 17})
        setdefault(jsonasobj_repr, 'z', {"a": 17})
        self.assertEqual(json_repr['z']['a'], jsonasobj_repr.z.a)
        self.assertEqual(list(keys(json_repr)), list(keys(jsonasobj_repr)))
        # TODO: this test should probably be finished -- we still have list of JsonObj issues
        # self.assertEqual(list(items(json_repr)),
        #                  [(k, as_json_obj(v) if isinstance(v, JsonObj) else v) for k, v in items(jsonasobj_repr)])

    def test_idempotent(self):
        """ JsonObj should be idempotent """
        o = JsonObj({"a": 42})
        self.assertEqual(id(o), id(JsonObj(o)))
        self.assertNotEqual(id(o), id(JsonObj(o, _if_missing=1)))


if __name__ == '__main__':
    unittest.main()




import unittest


class ByteArrayTestCase(unittest.TestCase):
    def test_bytearray(self):
        from jsonasobj import load
        load("http://hl7.org/fhir/R4/patient-example-f201-roel.json")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import sys
from setuptools import setup
from warnings import warn

if sys.version_info < (3, 6, 0):
    warn(f"jsonasobj requires python 3.6.0 or later.  Current version: {sys.version_info}")

setup(
    setup_requires=['pbr'],
    pbr=True,
)




import unittest
import jsonasobj

test_json = """{
    "k1": 1,
    "k2": "abc",
    "k3": {
        "x1": "foo",
        "x2": 17
    },
    "k4": [1, "abc", {"k5": 42}]
}"""


class GetTestCase(unittest.TestCase):
    def test_get(self):
        py_obj = jsonasobj.loads(test_json)
        self.assertEqual(1, py_obj.k1)
        with self.assertRaises(AttributeError):
            py_obj.e1
        self.assertIsNone(py_obj._get('e1'))
        self.assertEqual("abc", py_obj._get('e2', 'abc'))

if __name__ == '__main__':
    unittest.main()




import unittest

import os

from jsonasobj import JsonObj


class LoadTestCase(unittest.TestCase):
    def test_load_file(self):
        from jsonasobj import load
        json_fname = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'file.json')
        json_obj = load(json_fname)
        self.assertEqual([1, False, -12.7, "qwert"], json_obj.a_dict.vals)

    def test_load_uri(self):
        from jsonasobj import load
        # A relatively stable JSON file
        json_obj = load("http://hl7.org/fhir/STU3/account-example.json")
        self.assertEqual('Coverage/7546D', json_obj.coverage[0].coverage.reference)

    def test_load_fp(self):
        from jsonasobj import load
        json_fname = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'file.json')
        with open(json_fname) as f:
            json_obj = load(f)
        self.assertEqual([1, False, -12.7, "qwert"], json_obj.a_dict.vals)

    def test_bad_load(self):
        from jsonasobj import load
        json_fname = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'filex.json')
        with self.assertRaises(FileNotFoundError):
            json_obj = load(json_fname)
        self.assertEqual(JsonObj(), load(dict()))


    @unittest.skipIf(True, "FHIR servers no longer appear to do redirects (correctly)")
    def test_load_redirect(self):
        from jsonasobj import load
        json_obj = load("http://hl7.org/fhir/Patient/f001")
        self.assertEqual('male', json_obj.gender)

if __name__ == '__main__':
    unittest.main()



import unittest

from jsonasobj import JsonObj, as_json, as_dict

indict = {
    "a": "a",
    "b": {
        "c": "x",
        "d": 1
    }
}

class NestedDictTestCase(unittest.TestCase):

    def test_issue_8(self):
        """ Inner elements aren't loaded as JSON objects """
        injson = JsonObj(indict)
        self.assertTrue(isinstance(injson.b, JsonObj))
        self.assertEqual(indict, as_dict(injson))


if __name__ == '__main__':
    unittest.main()
